package bookExamples.ch26Graphics.draw2d;

import gui.ClosableJFrame;
import gui.run.RunMenu;
import gui.run.RunMenuBar;
import gui.run.RunMenuItem;
import j2d.ImageUtils;

import javax.swing.*;
import java.awt.*;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;

public class JDrawPanel extends JPanel
        implements MouseListener, MouseMotionListener {
    Font font = new Font("Times", Font.PLAIN, 12);


    private RunMenuItem line_mi = new RunMenuItem("[l]ine") {
        public void run() {
            lastSelected = line_mi;
        }
    };

    private RunMenuItem circle_mi = new RunMenuItem("[c]ircle") {
        public void run() {
            lastSelected = circle_mi;
        }
    };


    private RunMenuItem oval_mi = new RunMenuItem("[o]val") {
        public void run() {
            lastSelected = oval_mi;
        }
    };

    private RunMenuItem rect_mi = new RunMenuItem("rect") {
        public void run() {
            rect();
        }
    };

    private RunMenuItem string_mi = new RunMenuItem("[string{control s}") {
        public void run() {
            string();
        }
    };

    private RunMenuItem ray_mi = new RunMenuItem("ray") {
        public void run() {
            ray();
        }
    };

    private RunMenuItem lens_mi = new RunMenuItem("lens") {
        public void run() {
            lens();
        }
    };


    private RunMenuItem erase_mi = new RunMenuItem("erase") {
        public void run() {
            erase();
        }
    };


    private RunMenuItem lastSelected = line_mi;
    private Shapes shapeList = new Shapes();
    private int x1, y1;
    private int x2, y2;
    private int xl, yl;

    JDrawPanel() {
        addMouseMotionListener(this);
        addMouseListener(this);
        setLayout(new java.awt.FlowLayout());

    }

    private RunMenuBar getMenuBar() {
        RunMenuBar mb = new RunMenuBar();
        mb.add(getFileMenu());
        return mb;
    }

    private RunMenu getFileMenu() {
        RunMenu fileMenu = new RunMenu("[File");
        fileMenu.add(new RunMenuItem("ave as pict{control s}") {
            public void run() {
                save();
            }
        });
        fileMenu.add(new RunMenuItem("[save as pict{alt control s}") {
            public void run() {
                save();
            }
        });
        fileMenu.add(lens_mi);
        fileMenu.add(erase_mi);
        return fileMenu;
    }

    private void save() {
        ImageUtils.print(this);
    }

    private void rect() {
        lastSelected = rect_mi;
    }

    private void string() {
        lastSelected = string_mi;
    }

    private void lens() {
        lastSelected = lens_mi;
    }

    private void ray() {
        shapeList.addLineToLastCircle();
        repaint();
    }

    private void erase() {
        shapeList = new Shapes();
        repaint();
    }


    public void mouseDragged(MouseEvent e) {
        e.consume();
        xl = x2;
        yl = y2;
        x2 = e.getX();
        y2 = e.getY();
        repaint();
    }

    public void mouseMoved(MouseEvent e) {
    }

    public void mousePressed(MouseEvent e) {
        e.consume();
        x1 = e.getX();
        y1 = e.getY();
        int x21 = -1;
        x2 = x21;
    }

    public void mouseReleased(MouseEvent e) {
        e.consume();
        if (lastSelected == lens_mi) shapeList.add(new Arc2d(x1, y1, e.getX(), e.getY()));
        if (lastSelected == string_mi)
            shapeList.add(new String2d(e.getX(), e.getY(), "String me!"));
        if (lastSelected == line_mi) shapeList.add(new Line2d(x1, y1, e.getX(), e.getY()));
        if (lastSelected == rect_mi) shapeList.add(new Rect2d(x1, y1, e.getX(), e.getY()));
        if (lastSelected == circle_mi) shapeList.add(new Circle2d(x1, y1, e.getX(), e.getY()));
        if (lastSelected == oval_mi) shapeList.add(new Oval2d(x1, y1, e.getX(), e.getY()));
        int xl1 = -1;
        xl = xl1;
        int x21 = -1;
        x2 = x21;
        repaint();
    }

    public void mouseEntered(MouseEvent e) {
    }

    public void mouseExited(MouseEvent e) {
    }

    public void mouseClicked(MouseEvent e) {
    }

    public void drawRubberBand(Graphics g) {
        if (x2 == -1) return;
        g.setXORMode(getBackground());
        if (lastSelected == lens_mi) {
            Arc2d a =
                    new Arc2d(x1, y1, x2, y2);
            a.paint(g);
            return;
        }
        if (lastSelected == string_mi) {
            String2d s =
                    new String2d(x2, y2, "String me!");
            s.paint(g);
            return;
        }
        if (lastSelected == line_mi) {
            g.drawLine(x1, y1, x2, y2);
            return;
        }
        if (lastSelected == circle_mi) {
            Circle2d c = new Circle2d(x1, y1, x2, y2);
            c.paint(g);
            return;
        }
        if (lastSelected == oval_mi) {
            Oval2d o = new Oval2d(x1, y1, x2, y2);
            o.paint(g);
            return;
        }
        if (lastSelected == rect_mi) {
            Rect2d r = new Rect2d(x1, y1, x2, y2);
            r.paint(g);
            return;
        }
    }

    public void paint(Graphics g) {
        g.setFont(font);
        shapeList.paint(g);
        drawRubberBand(g);
    }

    public int getXl() {
        return xl;
    }

    public int getYl() {
        return yl;
    }

    public static void main(String args[]) {
        JDrawPanel drawPanel = new JDrawPanel();
        drawPanel.setSize(300, 300);
        ClosableJFrame cf =new ClosableJFrame();
        Container c = cf.getContentPane();
        c.add(drawPanel);
        c.setLayout(new FlowLayout());
        cf.setJMenuBar(drawPanel.getMenuBar());
        cf.setSize(300,300);
        cf.setVisible(true);
    }


}













